# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: events.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import meta_pb2 as meta__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='events.proto',
  package='voltha',
  syntax='proto3',
  serialized_pb=_b('\n\x0c\x65vents.proto\x12\x06voltha\x1a\nmeta.proto\x1a\x1cgoogle/api/annotations.proto\"E\n\x0f\x43onfigEventType\"2\n\x0f\x43onfigEventType\x12\x07\n\x03\x61\x64\x64\x10\x00\x12\n\n\x06remove\x10\x01\x12\n\n\x06update\x10\x02\"`\n\x0b\x43onfigEvent\x12\x35\n\x04type\x18\x01 \x01(\x0e\x32\'.voltha.ConfigEventType.ConfigEventType\x12\x0c\n\x04hash\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\t\"1\n\x0cKpiEventType\"!\n\x0cKpiEventType\x12\t\n\x05slice\x10\x00\x12\x06\n\x02ts\x10\x01\"\xd2\x01\n\x0eMetricMetaData\x12\r\n\x05title\x18\x01 \x01(\t\x12\n\n\x02ts\x18\x02 \x01(\x01\x12\x19\n\x11logical_device_id\x18\x03 \x01(\t\x12\x11\n\tserial_no\x18\x04 \x01(\t\x12\x11\n\tdevice_id\x18\x05 \x01(\t\x12\x34\n\x07\x63ontext\x18\x06 \x03(\x0b\x32#.voltha.MetricMetaData.ContextEntry\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"z\n\x10MetricValuePairs\x12\x36\n\x07metrics\x18\x01 \x03(\x0b\x32%.voltha.MetricValuePairs.MetricsEntry\x1a.\n\x0cMetricsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"\xa6\x01\n\x11MetricInformation\x12(\n\x08metadata\x18\x01 \x01(\x0b\x32\x16.voltha.MetricMetaData\x12\x37\n\x07metrics\x18\x02 \x03(\x0b\x32&.voltha.MetricInformation.MetricsEntry\x1a.\n\x0cMetricsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"\xc4\x01\n\x08KpiEvent\x12/\n\x04type\x18\x01 \x01(\x0e\x32!.voltha.KpiEventType.KpiEventType\x12\n\n\x02ts\x18\x02 \x01(\x02\x12\x30\n\x08prefixes\x18\x03 \x03(\x0b\x32\x1e.voltha.KpiEvent.PrefixesEntry\x1aI\n\rPrefixesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.voltha.MetricValuePairs:\x02\x38\x01\"w\n\tKpiEvent2\x12/\n\x04type\x18\x01 \x01(\x0e\x32!.voltha.KpiEventType.KpiEventType\x12\n\n\x02ts\x18\x02 \x01(\x01\x12-\n\nslice_data\x18\x03 \x03(\x0b\x32\x19.voltha.MetricInformation\"\x80\x01\n\x0e\x41larmEventType\"n\n\x0e\x41larmEventType\x12\x11\n\rCOMMUNICATION\x10\x00\x12\x0f\n\x0b\x45NVIRONMENT\x10\x01\x12\r\n\tEQUIPMENT\x10\x02\x12\x0b\n\x07SERVICE\x10\x03\x12\x0e\n\nPROCESSING\x10\x04\x12\x0c\n\x08SECURITY\x10\x05\"W\n\x12\x41larmEventCategory\"A\n\x12\x41larmEventCategory\x12\x07\n\x03PON\x10\x00\x12\x07\n\x03OLT\x10\x01\x12\x07\n\x03ONT\x10\x02\x12\x07\n\x03ONU\x10\x03\x12\x07\n\x03NNI\x10\x04\"=\n\x0f\x41larmEventState\"*\n\x0f\x41larmEventState\x12\n\n\x06RAISED\x10\x00\x12\x0b\n\x07\x43LEARED\x10\x01\"n\n\x12\x41larmEventSeverity\"X\n\x12\x41larmEventSeverity\x12\x11\n\rINDETERMINATE\x10\x00\x12\x0b\n\x07WARNING\x10\x01\x12\t\n\x05MINOR\x10\x02\x12\t\n\x05MAJOR\x10\x03\x12\x0c\n\x08\x43RITICAL\x10\x04\"\x83\x04\n\nAlarmEvent\x12\n\n\x02id\x18\x01 \x01(\t\x12\x33\n\x04type\x18\x02 \x01(\x0e\x32%.voltha.AlarmEventType.AlarmEventType\x12?\n\x08\x63\x61tegory\x18\x03 \x01(\x0e\x32-.voltha.AlarmEventCategory.AlarmEventCategory\x12\x36\n\x05state\x18\x04 \x01(\x0e\x32\'.voltha.AlarmEventState.AlarmEventState\x12?\n\x08severity\x18\x05 \x01(\x0e\x32-.voltha.AlarmEventSeverity.AlarmEventSeverity\x12\x11\n\traised_ts\x18\x06 \x01(\x02\x12\x13\n\x0breported_ts\x18\x07 \x01(\x02\x12\x12\n\nchanged_ts\x18\x08 \x01(\x02\x12\x13\n\x0bresource_id\x18\t \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\n \x01(\t\x12\x30\n\x07\x63ontext\x18\x0b \x03(\x0b\x32\x1f.voltha.AlarmEvent.ContextEntry\x12\x19\n\x11logical_device_id\x18\x0c \x01(\t\x12\x17\n\x0f\x61larm_type_name\x18\r \x01(\t\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42-Z+github.com/opencord/voltha-go/protos/volthab\x06proto3')
  ,
  dependencies=[meta__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])



_CONFIGEVENTTYPE_CONFIGEVENTTYPE = _descriptor.EnumDescriptor(
  name='ConfigEventType',
  full_name='voltha.ConfigEventType.ConfigEventType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='add', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='remove', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='update', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=85,
  serialized_end=135,
)
_sym_db.RegisterEnumDescriptor(_CONFIGEVENTTYPE_CONFIGEVENTTYPE)

_KPIEVENTTYPE_KPIEVENTTYPE = _descriptor.EnumDescriptor(
  name='KpiEventType',
  full_name='voltha.KpiEventType.KpiEventType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='slice', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ts', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=251,
  serialized_end=284,
)
_sym_db.RegisterEnumDescriptor(_KPIEVENTTYPE_KPIEVENTTYPE)

_ALARMEVENTTYPE_ALARMEVENTTYPE = _descriptor.EnumDescriptor(
  name='AlarmEventType',
  full_name='voltha.AlarmEventType.AlarmEventType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='COMMUNICATION', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ENVIRONMENT', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EQUIPMENT', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SERVICE', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROCESSING', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SECURITY', index=5, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1131,
  serialized_end=1241,
)
_sym_db.RegisterEnumDescriptor(_ALARMEVENTTYPE_ALARMEVENTTYPE)

_ALARMEVENTCATEGORY_ALARMEVENTCATEGORY = _descriptor.EnumDescriptor(
  name='AlarmEventCategory',
  full_name='voltha.AlarmEventCategory.AlarmEventCategory',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PON', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OLT', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ONT', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ONU', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NNI', index=4, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1265,
  serialized_end=1330,
)
_sym_db.RegisterEnumDescriptor(_ALARMEVENTCATEGORY_ALARMEVENTCATEGORY)

_ALARMEVENTSTATE_ALARMEVENTSTATE = _descriptor.EnumDescriptor(
  name='AlarmEventState',
  full_name='voltha.AlarmEventState.AlarmEventState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RAISED', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLEARED', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1351,
  serialized_end=1393,
)
_sym_db.RegisterEnumDescriptor(_ALARMEVENTSTATE_ALARMEVENTSTATE)

_ALARMEVENTSEVERITY_ALARMEVENTSEVERITY = _descriptor.EnumDescriptor(
  name='AlarmEventSeverity',
  full_name='voltha.AlarmEventSeverity.AlarmEventSeverity',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INDETERMINATE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WARNING', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MINOR', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MAJOR', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CRITICAL', index=4, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1417,
  serialized_end=1505,
)
_sym_db.RegisterEnumDescriptor(_ALARMEVENTSEVERITY_ALARMEVENTSEVERITY)


_CONFIGEVENTTYPE = _descriptor.Descriptor(
  name='ConfigEventType',
  full_name='voltha.ConfigEventType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _CONFIGEVENTTYPE_CONFIGEVENTTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=135,
)


_CONFIGEVENT = _descriptor.Descriptor(
  name='ConfigEvent',
  full_name='voltha.ConfigEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='voltha.ConfigEvent.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hash', full_name='voltha.ConfigEvent.hash', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='data', full_name='voltha.ConfigEvent.data', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=233,
)


_KPIEVENTTYPE = _descriptor.Descriptor(
  name='KpiEventType',
  full_name='voltha.KpiEventType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _KPIEVENTTYPE_KPIEVENTTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=235,
  serialized_end=284,
)


_METRICMETADATA_CONTEXTENTRY = _descriptor.Descriptor(
  name='ContextEntry',
  full_name='voltha.MetricMetaData.ContextEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='voltha.MetricMetaData.ContextEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='voltha.MetricMetaData.ContextEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=451,
  serialized_end=497,
)

_METRICMETADATA = _descriptor.Descriptor(
  name='MetricMetaData',
  full_name='voltha.MetricMetaData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='voltha.MetricMetaData.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ts', full_name='voltha.MetricMetaData.ts', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='logical_device_id', full_name='voltha.MetricMetaData.logical_device_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='serial_no', full_name='voltha.MetricMetaData.serial_no', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_id', full_name='voltha.MetricMetaData.device_id', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='context', full_name='voltha.MetricMetaData.context', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_METRICMETADATA_CONTEXTENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=287,
  serialized_end=497,
)


_METRICVALUEPAIRS_METRICSENTRY = _descriptor.Descriptor(
  name='MetricsEntry',
  full_name='voltha.MetricValuePairs.MetricsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='voltha.MetricValuePairs.MetricsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='voltha.MetricValuePairs.MetricsEntry.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=575,
  serialized_end=621,
)

_METRICVALUEPAIRS = _descriptor.Descriptor(
  name='MetricValuePairs',
  full_name='voltha.MetricValuePairs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metrics', full_name='voltha.MetricValuePairs.metrics', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_METRICVALUEPAIRS_METRICSENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=499,
  serialized_end=621,
)


_METRICINFORMATION_METRICSENTRY = _descriptor.Descriptor(
  name='MetricsEntry',
  full_name='voltha.MetricInformation.MetricsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='voltha.MetricInformation.MetricsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='voltha.MetricInformation.MetricsEntry.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=575,
  serialized_end=621,
)

_METRICINFORMATION = _descriptor.Descriptor(
  name='MetricInformation',
  full_name='voltha.MetricInformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='voltha.MetricInformation.metadata', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metrics', full_name='voltha.MetricInformation.metrics', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_METRICINFORMATION_METRICSENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=624,
  serialized_end=790,
)


_KPIEVENT_PREFIXESENTRY = _descriptor.Descriptor(
  name='PrefixesEntry',
  full_name='voltha.KpiEvent.PrefixesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='voltha.KpiEvent.PrefixesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='voltha.KpiEvent.PrefixesEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=916,
  serialized_end=989,
)

_KPIEVENT = _descriptor.Descriptor(
  name='KpiEvent',
  full_name='voltha.KpiEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='voltha.KpiEvent.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ts', full_name='voltha.KpiEvent.ts', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prefixes', full_name='voltha.KpiEvent.prefixes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_KPIEVENT_PREFIXESENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=793,
  serialized_end=989,
)


_KPIEVENT2 = _descriptor.Descriptor(
  name='KpiEvent2',
  full_name='voltha.KpiEvent2',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='voltha.KpiEvent2.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ts', full_name='voltha.KpiEvent2.ts', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='slice_data', full_name='voltha.KpiEvent2.slice_data', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=991,
  serialized_end=1110,
)


_ALARMEVENTTYPE = _descriptor.Descriptor(
  name='AlarmEventType',
  full_name='voltha.AlarmEventType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ALARMEVENTTYPE_ALARMEVENTTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1113,
  serialized_end=1241,
)


_ALARMEVENTCATEGORY = _descriptor.Descriptor(
  name='AlarmEventCategory',
  full_name='voltha.AlarmEventCategory',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ALARMEVENTCATEGORY_ALARMEVENTCATEGORY,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1243,
  serialized_end=1330,
)


_ALARMEVENTSTATE = _descriptor.Descriptor(
  name='AlarmEventState',
  full_name='voltha.AlarmEventState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ALARMEVENTSTATE_ALARMEVENTSTATE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1332,
  serialized_end=1393,
)


_ALARMEVENTSEVERITY = _descriptor.Descriptor(
  name='AlarmEventSeverity',
  full_name='voltha.AlarmEventSeverity',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ALARMEVENTSEVERITY_ALARMEVENTSEVERITY,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1395,
  serialized_end=1505,
)


_ALARMEVENT_CONTEXTENTRY = _descriptor.Descriptor(
  name='ContextEntry',
  full_name='voltha.AlarmEvent.ContextEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='voltha.AlarmEvent.ContextEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='voltha.AlarmEvent.ContextEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=451,
  serialized_end=497,
)

_ALARMEVENT = _descriptor.Descriptor(
  name='AlarmEvent',
  full_name='voltha.AlarmEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='voltha.AlarmEvent.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='voltha.AlarmEvent.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='category', full_name='voltha.AlarmEvent.category', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='voltha.AlarmEvent.state', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='severity', full_name='voltha.AlarmEvent.severity', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='raised_ts', full_name='voltha.AlarmEvent.raised_ts', index=5,
      number=6, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reported_ts', full_name='voltha.AlarmEvent.reported_ts', index=6,
      number=7, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='changed_ts', full_name='voltha.AlarmEvent.changed_ts', index=7,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='resource_id', full_name='voltha.AlarmEvent.resource_id', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='voltha.AlarmEvent.description', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='context', full_name='voltha.AlarmEvent.context', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='logical_device_id', full_name='voltha.AlarmEvent.logical_device_id', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='alarm_type_name', full_name='voltha.AlarmEvent.alarm_type_name', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_ALARMEVENT_CONTEXTENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1508,
  serialized_end=2023,
)

_CONFIGEVENTTYPE_CONFIGEVENTTYPE.containing_type = _CONFIGEVENTTYPE
_CONFIGEVENT.fields_by_name['type'].enum_type = _CONFIGEVENTTYPE_CONFIGEVENTTYPE
_KPIEVENTTYPE_KPIEVENTTYPE.containing_type = _KPIEVENTTYPE
_METRICMETADATA_CONTEXTENTRY.containing_type = _METRICMETADATA
_METRICMETADATA.fields_by_name['context'].message_type = _METRICMETADATA_CONTEXTENTRY
_METRICVALUEPAIRS_METRICSENTRY.containing_type = _METRICVALUEPAIRS
_METRICVALUEPAIRS.fields_by_name['metrics'].message_type = _METRICVALUEPAIRS_METRICSENTRY
_METRICINFORMATION_METRICSENTRY.containing_type = _METRICINFORMATION
_METRICINFORMATION.fields_by_name['metadata'].message_type = _METRICMETADATA
_METRICINFORMATION.fields_by_name['metrics'].message_type = _METRICINFORMATION_METRICSENTRY
_KPIEVENT_PREFIXESENTRY.fields_by_name['value'].message_type = _METRICVALUEPAIRS
_KPIEVENT_PREFIXESENTRY.containing_type = _KPIEVENT
_KPIEVENT.fields_by_name['type'].enum_type = _KPIEVENTTYPE_KPIEVENTTYPE
_KPIEVENT.fields_by_name['prefixes'].message_type = _KPIEVENT_PREFIXESENTRY
_KPIEVENT2.fields_by_name['type'].enum_type = _KPIEVENTTYPE_KPIEVENTTYPE
_KPIEVENT2.fields_by_name['slice_data'].message_type = _METRICINFORMATION
_ALARMEVENTTYPE_ALARMEVENTTYPE.containing_type = _ALARMEVENTTYPE
_ALARMEVENTCATEGORY_ALARMEVENTCATEGORY.containing_type = _ALARMEVENTCATEGORY
_ALARMEVENTSTATE_ALARMEVENTSTATE.containing_type = _ALARMEVENTSTATE
_ALARMEVENTSEVERITY_ALARMEVENTSEVERITY.containing_type = _ALARMEVENTSEVERITY
_ALARMEVENT_CONTEXTENTRY.containing_type = _ALARMEVENT
_ALARMEVENT.fields_by_name['type'].enum_type = _ALARMEVENTTYPE_ALARMEVENTTYPE
_ALARMEVENT.fields_by_name['category'].enum_type = _ALARMEVENTCATEGORY_ALARMEVENTCATEGORY
_ALARMEVENT.fields_by_name['state'].enum_type = _ALARMEVENTSTATE_ALARMEVENTSTATE
_ALARMEVENT.fields_by_name['severity'].enum_type = _ALARMEVENTSEVERITY_ALARMEVENTSEVERITY
_ALARMEVENT.fields_by_name['context'].message_type = _ALARMEVENT_CONTEXTENTRY
DESCRIPTOR.message_types_by_name['ConfigEventType'] = _CONFIGEVENTTYPE
DESCRIPTOR.message_types_by_name['ConfigEvent'] = _CONFIGEVENT
DESCRIPTOR.message_types_by_name['KpiEventType'] = _KPIEVENTTYPE
DESCRIPTOR.message_types_by_name['MetricMetaData'] = _METRICMETADATA
DESCRIPTOR.message_types_by_name['MetricValuePairs'] = _METRICVALUEPAIRS
DESCRIPTOR.message_types_by_name['MetricInformation'] = _METRICINFORMATION
DESCRIPTOR.message_types_by_name['KpiEvent'] = _KPIEVENT
DESCRIPTOR.message_types_by_name['KpiEvent2'] = _KPIEVENT2
DESCRIPTOR.message_types_by_name['AlarmEventType'] = _ALARMEVENTTYPE
DESCRIPTOR.message_types_by_name['AlarmEventCategory'] = _ALARMEVENTCATEGORY
DESCRIPTOR.message_types_by_name['AlarmEventState'] = _ALARMEVENTSTATE
DESCRIPTOR.message_types_by_name['AlarmEventSeverity'] = _ALARMEVENTSEVERITY
DESCRIPTOR.message_types_by_name['AlarmEvent'] = _ALARMEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ConfigEventType = _reflection.GeneratedProtocolMessageType('ConfigEventType', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGEVENTTYPE,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.ConfigEventType)
  ))
_sym_db.RegisterMessage(ConfigEventType)

ConfigEvent = _reflection.GeneratedProtocolMessageType('ConfigEvent', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGEVENT,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.ConfigEvent)
  ))
_sym_db.RegisterMessage(ConfigEvent)

KpiEventType = _reflection.GeneratedProtocolMessageType('KpiEventType', (_message.Message,), dict(
  DESCRIPTOR = _KPIEVENTTYPE,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.KpiEventType)
  ))
_sym_db.RegisterMessage(KpiEventType)

MetricMetaData = _reflection.GeneratedProtocolMessageType('MetricMetaData', (_message.Message,), dict(

  ContextEntry = _reflection.GeneratedProtocolMessageType('ContextEntry', (_message.Message,), dict(
    DESCRIPTOR = _METRICMETADATA_CONTEXTENTRY,
    __module__ = 'events_pb2'
    # @@protoc_insertion_point(class_scope:voltha.MetricMetaData.ContextEntry)
    ))
  ,
  DESCRIPTOR = _METRICMETADATA,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.MetricMetaData)
  ))
_sym_db.RegisterMessage(MetricMetaData)
_sym_db.RegisterMessage(MetricMetaData.ContextEntry)

MetricValuePairs = _reflection.GeneratedProtocolMessageType('MetricValuePairs', (_message.Message,), dict(

  MetricsEntry = _reflection.GeneratedProtocolMessageType('MetricsEntry', (_message.Message,), dict(
    DESCRIPTOR = _METRICVALUEPAIRS_METRICSENTRY,
    __module__ = 'events_pb2'
    # @@protoc_insertion_point(class_scope:voltha.MetricValuePairs.MetricsEntry)
    ))
  ,
  DESCRIPTOR = _METRICVALUEPAIRS,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.MetricValuePairs)
  ))
_sym_db.RegisterMessage(MetricValuePairs)
_sym_db.RegisterMessage(MetricValuePairs.MetricsEntry)

MetricInformation = _reflection.GeneratedProtocolMessageType('MetricInformation', (_message.Message,), dict(

  MetricsEntry = _reflection.GeneratedProtocolMessageType('MetricsEntry', (_message.Message,), dict(
    DESCRIPTOR = _METRICINFORMATION_METRICSENTRY,
    __module__ = 'events_pb2'
    # @@protoc_insertion_point(class_scope:voltha.MetricInformation.MetricsEntry)
    ))
  ,
  DESCRIPTOR = _METRICINFORMATION,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.MetricInformation)
  ))
_sym_db.RegisterMessage(MetricInformation)
_sym_db.RegisterMessage(MetricInformation.MetricsEntry)

KpiEvent = _reflection.GeneratedProtocolMessageType('KpiEvent', (_message.Message,), dict(

  PrefixesEntry = _reflection.GeneratedProtocolMessageType('PrefixesEntry', (_message.Message,), dict(
    DESCRIPTOR = _KPIEVENT_PREFIXESENTRY,
    __module__ = 'events_pb2'
    # @@protoc_insertion_point(class_scope:voltha.KpiEvent.PrefixesEntry)
    ))
  ,
  DESCRIPTOR = _KPIEVENT,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.KpiEvent)
  ))
_sym_db.RegisterMessage(KpiEvent)
_sym_db.RegisterMessage(KpiEvent.PrefixesEntry)

KpiEvent2 = _reflection.GeneratedProtocolMessageType('KpiEvent2', (_message.Message,), dict(
  DESCRIPTOR = _KPIEVENT2,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.KpiEvent2)
  ))
_sym_db.RegisterMessage(KpiEvent2)

AlarmEventType = _reflection.GeneratedProtocolMessageType('AlarmEventType', (_message.Message,), dict(
  DESCRIPTOR = _ALARMEVENTTYPE,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmEventType)
  ))
_sym_db.RegisterMessage(AlarmEventType)

AlarmEventCategory = _reflection.GeneratedProtocolMessageType('AlarmEventCategory', (_message.Message,), dict(
  DESCRIPTOR = _ALARMEVENTCATEGORY,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmEventCategory)
  ))
_sym_db.RegisterMessage(AlarmEventCategory)

AlarmEventState = _reflection.GeneratedProtocolMessageType('AlarmEventState', (_message.Message,), dict(
  DESCRIPTOR = _ALARMEVENTSTATE,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmEventState)
  ))
_sym_db.RegisterMessage(AlarmEventState)

AlarmEventSeverity = _reflection.GeneratedProtocolMessageType('AlarmEventSeverity', (_message.Message,), dict(
  DESCRIPTOR = _ALARMEVENTSEVERITY,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmEventSeverity)
  ))
_sym_db.RegisterMessage(AlarmEventSeverity)

AlarmEvent = _reflection.GeneratedProtocolMessageType('AlarmEvent', (_message.Message,), dict(

  ContextEntry = _reflection.GeneratedProtocolMessageType('ContextEntry', (_message.Message,), dict(
    DESCRIPTOR = _ALARMEVENT_CONTEXTENTRY,
    __module__ = 'events_pb2'
    # @@protoc_insertion_point(class_scope:voltha.AlarmEvent.ContextEntry)
    ))
  ,
  DESCRIPTOR = _ALARMEVENT,
  __module__ = 'events_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmEvent)
  ))
_sym_db.RegisterMessage(AlarmEvent)
_sym_db.RegisterMessage(AlarmEvent.ContextEntry)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z+github.com/opencord/voltha-go/protos/voltha'))
_METRICMETADATA_CONTEXTENTRY.has_options = True
_METRICMETADATA_CONTEXTENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_METRICVALUEPAIRS_METRICSENTRY.has_options = True
_METRICVALUEPAIRS_METRICSENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_METRICINFORMATION_METRICSENTRY.has_options = True
_METRICINFORMATION_METRICSENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_KPIEVENT_PREFIXESENTRY.has_options = True
_KPIEVENT_PREFIXESENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_ALARMEVENT_CONTEXTENTRY.has_options = True
_ALARMEVENT_CONTEXTENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
