# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: inter_container.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
import openflow_13_pb2 as openflow__13__pb2
try:
  yang__options__pb2 = openflow__13__pb2.yang__options__pb2
except AttributeError:
  yang__options__pb2 = openflow__13__pb2.yang_options_pb2
import logical_device_pb2 as logical__device__pb2

from logical_device_pb2 import *

DESCRIPTOR = _descriptor.FileDescriptor(
  name='inter_container.proto',
  package='voltha',
  syntax='proto3',
  serialized_pb=_b('\n\x15inter_container.proto\x12\x06voltha\x1a\x19google/protobuf/any.proto\x1a\x11openflow_13.proto\x1a\x14logical_device.proto\"\x16\n\x07StrType\x12\x0b\n\x03val\x18\x01 \x01(\t\"\x16\n\x07IntType\x12\x0b\n\x03val\x18\x01 \x01(\x03\"\x17\n\x08\x42oolType\x12\x0b\n\x03val\x18\x01 \x01(\x08\"\x19\n\x06Packet\x12\x0f\n\x07payload\x18\x01 \x01(\x0c\"E\n\tErrorCode\"8\n\x05\x63odes\x12\x17\n\x13UNSUPPORTED_REQUEST\x10\x00\x12\x16\n\x12INVALID_PARAMETERS\x10\x01\"8\n\x05\x45rror\x12\x1f\n\x04\x63ode\x18\x01 \x01(\x0b\x32\x11.voltha.ErrorCode\x12\x0e\n\x06reason\x18\x02 \x01(\t\"p\n\x06Header\x12\n\n\x02id\x18\x01 \x01(\t\x12!\n\x04type\x18\x02 \x01(\x0e\x32\x13.voltha.MessageType\x12\x12\n\nfrom_topic\x18\x03 \x01(\t\x12\x10\n\x08to_topic\x18\x04 \x01(\t\x12\x11\n\ttimestamp\x18\x05 \x01(\x03\"<\n\x08\x41rgument\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any\"[\n\x15InterContainerMessage\x12\x1e\n\x06header\x18\x01 \x01(\x0b\x32\x0e.voltha.Header\x12\"\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any\"{\n\x19InterContainerRequestBody\x12\x0b\n\x03rpc\x18\x02 \x01(\t\x12\x1e\n\x04\x61rgs\x18\x03 \x03(\x0b\x32\x10.voltha.Argument\x12\x19\n\x11response_required\x18\x04 \x01(\x08\x12\x16\n\x0ereply_to_topic\x18\x05 \x01(\t\"S\n\x1aInterContainerResponseBody\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12$\n\x06result\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any\"r\n\x10SwitchCapability\x12#\n\x04\x64\x65sc\x18\x01 \x01(\x0b\x32\x15.openflow_13.ofp_desc\x12\x39\n\x0fswitch_features\x18\x02 \x01(\x0b\x32 .openflow_13.ofp_switch_features\"3\n\x0ePortCapability\x12!\n\x04port\x18\x01 \x01(\x0b\x32\x13.voltha.LogicalPort\"Y\n\x10\x44\x65viceDiscovered\x12\n\n\x02id\x18\x01 \x01(\t\x12\x11\n\tparent_id\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65vice_type\x18\x03 \x01(\t\x12\x11\n\tpublisher\x18\x04 \x01(\t\"\x97\x01\n\x17InterAdapterMessageType\"|\n\x05Types\x12\x10\n\x0c\x46LOW_REQUEST\x10\x00\x12\x11\n\rFLOW_RESPONSE\x10\x01\x12\x10\n\x0cOMCI_REQUEST\x10\x02\x12\x11\n\rOMCI_RESPONSE\x10\x03\x12\x13\n\x0fMETRICS_REQUEST\x10\x04\x12\x14\n\x10METRICS_RESPONSE\x10\x05\"\xbd\x01\n\x12InterAdapterHeader\x12\n\n\x02id\x18\x01 \x01(\t\x12\x33\n\x04type\x18\x02 \x01(\x0e\x32%.voltha.InterAdapterMessageType.Types\x12\x12\n\nfrom_topic\x18\x03 \x01(\t\x12\x10\n\x08to_topic\x18\x04 \x01(\t\x12\x14\n\x0cto_device_id\x18\x05 \x01(\t\x12\x17\n\x0fproxy_device_id\x18\x06 \x01(\t\x12\x11\n\ttimestamp\x18\x07 \x01(\x03\"N\n\x18InterAdapterResponseBody\x12\x0e\n\x06status\x18\x01 \x01(\x08\x12\"\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any\"e\n\x13InterAdapterMessage\x12*\n\x06header\x18\x01 \x01(\x0b\x32\x1a.voltha.InterAdapterHeader\x12\"\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any*?\n\x0bMessageType\x12\x0b\n\x07REQUEST\x10\x00\x12\x0c\n\x08RESPONSE\x10\x01\x12\x15\n\x11\x44\x45VICE_DISCOVERED\x10\x02\x42\x36Z4github.com/opencord/voltha-go/protos/inter_containerP\x02\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,openflow__13__pb2.DESCRIPTOR,logical__device__pb2.DESCRIPTOR,],
  public_dependencies=[logical__device__pb2.DESCRIPTOR,])

_MESSAGETYPE = _descriptor.EnumDescriptor(
  name='MessageType',
  full_name='voltha.MessageType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='REQUEST', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RESPONSE', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_DISCOVERED', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1598,
  serialized_end=1661,
)
_sym_db.RegisterEnumDescriptor(_MESSAGETYPE)

MessageType = enum_type_wrapper.EnumTypeWrapper(_MESSAGETYPE)
REQUEST = 0
RESPONSE = 1
DEVICE_DISCOVERED = 2


_ERRORCODE_CODES = _descriptor.EnumDescriptor(
  name='codes',
  full_name='voltha.ErrorCode.codes',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSUPPORTED_REQUEST', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_PARAMETERS', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=214,
  serialized_end=270,
)
_sym_db.RegisterEnumDescriptor(_ERRORCODE_CODES)

_INTERADAPTERMESSAGETYPE_TYPES = _descriptor.EnumDescriptor(
  name='Types',
  full_name='voltha.InterAdapterMessageType.Types',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FLOW_REQUEST', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FLOW_RESPONSE', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OMCI_REQUEST', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OMCI_RESPONSE', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='METRICS_REQUEST', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='METRICS_RESPONSE', index=5, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1097,
  serialized_end=1221,
)
_sym_db.RegisterEnumDescriptor(_INTERADAPTERMESSAGETYPE_TYPES)


_STRTYPE = _descriptor.Descriptor(
  name='StrType',
  full_name='voltha.StrType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='val', full_name='voltha.StrType.val', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=101,
  serialized_end=123,
)


_INTTYPE = _descriptor.Descriptor(
  name='IntType',
  full_name='voltha.IntType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='val', full_name='voltha.IntType.val', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=125,
  serialized_end=147,
)


_BOOLTYPE = _descriptor.Descriptor(
  name='BoolType',
  full_name='voltha.BoolType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='val', full_name='voltha.BoolType.val', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=149,
  serialized_end=172,
)


_PACKET = _descriptor.Descriptor(
  name='Packet',
  full_name='voltha.Packet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='payload', full_name='voltha.Packet.payload', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=174,
  serialized_end=199,
)


_ERRORCODE = _descriptor.Descriptor(
  name='ErrorCode',
  full_name='voltha.ErrorCode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ERRORCODE_CODES,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=201,
  serialized_end=270,
)


_ERROR = _descriptor.Descriptor(
  name='Error',
  full_name='voltha.Error',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='voltha.Error.code', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reason', full_name='voltha.Error.reason', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=272,
  serialized_end=328,
)


_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='voltha.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='voltha.Header.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='voltha.Header.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='from_topic', full_name='voltha.Header.from_topic', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='to_topic', full_name='voltha.Header.to_topic', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='voltha.Header.timestamp', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=330,
  serialized_end=442,
)


_ARGUMENT = _descriptor.Descriptor(
  name='Argument',
  full_name='voltha.Argument',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='voltha.Argument.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='voltha.Argument.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=444,
  serialized_end=504,
)


_INTERCONTAINERMESSAGE = _descriptor.Descriptor(
  name='InterContainerMessage',
  full_name='voltha.InterContainerMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='voltha.InterContainerMessage.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='body', full_name='voltha.InterContainerMessage.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=506,
  serialized_end=597,
)


_INTERCONTAINERREQUESTBODY = _descriptor.Descriptor(
  name='InterContainerRequestBody',
  full_name='voltha.InterContainerRequestBody',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rpc', full_name='voltha.InterContainerRequestBody.rpc', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='args', full_name='voltha.InterContainerRequestBody.args', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='response_required', full_name='voltha.InterContainerRequestBody.response_required', index=2,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reply_to_topic', full_name='voltha.InterContainerRequestBody.reply_to_topic', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=599,
  serialized_end=722,
)


_INTERCONTAINERRESPONSEBODY = _descriptor.Descriptor(
  name='InterContainerResponseBody',
  full_name='voltha.InterContainerResponseBody',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='voltha.InterContainerResponseBody.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='result', full_name='voltha.InterContainerResponseBody.result', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=724,
  serialized_end=807,
)


_SWITCHCAPABILITY = _descriptor.Descriptor(
  name='SwitchCapability',
  full_name='voltha.SwitchCapability',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='desc', full_name='voltha.SwitchCapability.desc', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='switch_features', full_name='voltha.SwitchCapability.switch_features', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=809,
  serialized_end=923,
)


_PORTCAPABILITY = _descriptor.Descriptor(
  name='PortCapability',
  full_name='voltha.PortCapability',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='port', full_name='voltha.PortCapability.port', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=925,
  serialized_end=976,
)


_DEVICEDISCOVERED = _descriptor.Descriptor(
  name='DeviceDiscovered',
  full_name='voltha.DeviceDiscovered',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='voltha.DeviceDiscovered.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parent_id', full_name='voltha.DeviceDiscovered.parent_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_type', full_name='voltha.DeviceDiscovered.device_type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='publisher', full_name='voltha.DeviceDiscovered.publisher', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=978,
  serialized_end=1067,
)


_INTERADAPTERMESSAGETYPE = _descriptor.Descriptor(
  name='InterAdapterMessageType',
  full_name='voltha.InterAdapterMessageType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _INTERADAPTERMESSAGETYPE_TYPES,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1070,
  serialized_end=1221,
)


_INTERADAPTERHEADER = _descriptor.Descriptor(
  name='InterAdapterHeader',
  full_name='voltha.InterAdapterHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='voltha.InterAdapterHeader.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='voltha.InterAdapterHeader.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='from_topic', full_name='voltha.InterAdapterHeader.from_topic', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='to_topic', full_name='voltha.InterAdapterHeader.to_topic', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='to_device_id', full_name='voltha.InterAdapterHeader.to_device_id', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='proxy_device_id', full_name='voltha.InterAdapterHeader.proxy_device_id', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='voltha.InterAdapterHeader.timestamp', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1224,
  serialized_end=1413,
)


_INTERADAPTERRESPONSEBODY = _descriptor.Descriptor(
  name='InterAdapterResponseBody',
  full_name='voltha.InterAdapterResponseBody',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='voltha.InterAdapterResponseBody.status', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='body', full_name='voltha.InterAdapterResponseBody.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1415,
  serialized_end=1493,
)


_INTERADAPTERMESSAGE = _descriptor.Descriptor(
  name='InterAdapterMessage',
  full_name='voltha.InterAdapterMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='voltha.InterAdapterMessage.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='body', full_name='voltha.InterAdapterMessage.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1495,
  serialized_end=1596,
)

_ERRORCODE_CODES.containing_type = _ERRORCODE
_ERROR.fields_by_name['code'].message_type = _ERRORCODE
_HEADER.fields_by_name['type'].enum_type = _MESSAGETYPE
_ARGUMENT.fields_by_name['value'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_INTERCONTAINERMESSAGE.fields_by_name['header'].message_type = _HEADER
_INTERCONTAINERMESSAGE.fields_by_name['body'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_INTERCONTAINERREQUESTBODY.fields_by_name['args'].message_type = _ARGUMENT
_INTERCONTAINERRESPONSEBODY.fields_by_name['result'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_SWITCHCAPABILITY.fields_by_name['desc'].message_type = openflow__13__pb2._OFP_DESC
_SWITCHCAPABILITY.fields_by_name['switch_features'].message_type = openflow__13__pb2._OFP_SWITCH_FEATURES
_PORTCAPABILITY.fields_by_name['port'].message_type = logical__device__pb2._LOGICALPORT
_INTERADAPTERMESSAGETYPE_TYPES.containing_type = _INTERADAPTERMESSAGETYPE
_INTERADAPTERHEADER.fields_by_name['type'].enum_type = _INTERADAPTERMESSAGETYPE_TYPES
_INTERADAPTERRESPONSEBODY.fields_by_name['body'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_INTERADAPTERMESSAGE.fields_by_name['header'].message_type = _INTERADAPTERHEADER
_INTERADAPTERMESSAGE.fields_by_name['body'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['StrType'] = _STRTYPE
DESCRIPTOR.message_types_by_name['IntType'] = _INTTYPE
DESCRIPTOR.message_types_by_name['BoolType'] = _BOOLTYPE
DESCRIPTOR.message_types_by_name['Packet'] = _PACKET
DESCRIPTOR.message_types_by_name['ErrorCode'] = _ERRORCODE
DESCRIPTOR.message_types_by_name['Error'] = _ERROR
DESCRIPTOR.message_types_by_name['Header'] = _HEADER
DESCRIPTOR.message_types_by_name['Argument'] = _ARGUMENT
DESCRIPTOR.message_types_by_name['InterContainerMessage'] = _INTERCONTAINERMESSAGE
DESCRIPTOR.message_types_by_name['InterContainerRequestBody'] = _INTERCONTAINERREQUESTBODY
DESCRIPTOR.message_types_by_name['InterContainerResponseBody'] = _INTERCONTAINERRESPONSEBODY
DESCRIPTOR.message_types_by_name['SwitchCapability'] = _SWITCHCAPABILITY
DESCRIPTOR.message_types_by_name['PortCapability'] = _PORTCAPABILITY
DESCRIPTOR.message_types_by_name['DeviceDiscovered'] = _DEVICEDISCOVERED
DESCRIPTOR.message_types_by_name['InterAdapterMessageType'] = _INTERADAPTERMESSAGETYPE
DESCRIPTOR.message_types_by_name['InterAdapterHeader'] = _INTERADAPTERHEADER
DESCRIPTOR.message_types_by_name['InterAdapterResponseBody'] = _INTERADAPTERRESPONSEBODY
DESCRIPTOR.message_types_by_name['InterAdapterMessage'] = _INTERADAPTERMESSAGE
DESCRIPTOR.enum_types_by_name['MessageType'] = _MESSAGETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StrType = _reflection.GeneratedProtocolMessageType('StrType', (_message.Message,), dict(
  DESCRIPTOR = _STRTYPE,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.StrType)
  ))
_sym_db.RegisterMessage(StrType)

IntType = _reflection.GeneratedProtocolMessageType('IntType', (_message.Message,), dict(
  DESCRIPTOR = _INTTYPE,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.IntType)
  ))
_sym_db.RegisterMessage(IntType)

BoolType = _reflection.GeneratedProtocolMessageType('BoolType', (_message.Message,), dict(
  DESCRIPTOR = _BOOLTYPE,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.BoolType)
  ))
_sym_db.RegisterMessage(BoolType)

Packet = _reflection.GeneratedProtocolMessageType('Packet', (_message.Message,), dict(
  DESCRIPTOR = _PACKET,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.Packet)
  ))
_sym_db.RegisterMessage(Packet)

ErrorCode = _reflection.GeneratedProtocolMessageType('ErrorCode', (_message.Message,), dict(
  DESCRIPTOR = _ERRORCODE,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.ErrorCode)
  ))
_sym_db.RegisterMessage(ErrorCode)

Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), dict(
  DESCRIPTOR = _ERROR,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.Error)
  ))
_sym_db.RegisterMessage(Error)

Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), dict(
  DESCRIPTOR = _HEADER,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.Header)
  ))
_sym_db.RegisterMessage(Header)

Argument = _reflection.GeneratedProtocolMessageType('Argument', (_message.Message,), dict(
  DESCRIPTOR = _ARGUMENT,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.Argument)
  ))
_sym_db.RegisterMessage(Argument)

InterContainerMessage = _reflection.GeneratedProtocolMessageType('InterContainerMessage', (_message.Message,), dict(
  DESCRIPTOR = _INTERCONTAINERMESSAGE,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.InterContainerMessage)
  ))
_sym_db.RegisterMessage(InterContainerMessage)

InterContainerRequestBody = _reflection.GeneratedProtocolMessageType('InterContainerRequestBody', (_message.Message,), dict(
  DESCRIPTOR = _INTERCONTAINERREQUESTBODY,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.InterContainerRequestBody)
  ))
_sym_db.RegisterMessage(InterContainerRequestBody)

InterContainerResponseBody = _reflection.GeneratedProtocolMessageType('InterContainerResponseBody', (_message.Message,), dict(
  DESCRIPTOR = _INTERCONTAINERRESPONSEBODY,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.InterContainerResponseBody)
  ))
_sym_db.RegisterMessage(InterContainerResponseBody)

SwitchCapability = _reflection.GeneratedProtocolMessageType('SwitchCapability', (_message.Message,), dict(
  DESCRIPTOR = _SWITCHCAPABILITY,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.SwitchCapability)
  ))
_sym_db.RegisterMessage(SwitchCapability)

PortCapability = _reflection.GeneratedProtocolMessageType('PortCapability', (_message.Message,), dict(
  DESCRIPTOR = _PORTCAPABILITY,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.PortCapability)
  ))
_sym_db.RegisterMessage(PortCapability)

DeviceDiscovered = _reflection.GeneratedProtocolMessageType('DeviceDiscovered', (_message.Message,), dict(
  DESCRIPTOR = _DEVICEDISCOVERED,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.DeviceDiscovered)
  ))
_sym_db.RegisterMessage(DeviceDiscovered)

InterAdapterMessageType = _reflection.GeneratedProtocolMessageType('InterAdapterMessageType', (_message.Message,), dict(
  DESCRIPTOR = _INTERADAPTERMESSAGETYPE,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.InterAdapterMessageType)
  ))
_sym_db.RegisterMessage(InterAdapterMessageType)

InterAdapterHeader = _reflection.GeneratedProtocolMessageType('InterAdapterHeader', (_message.Message,), dict(
  DESCRIPTOR = _INTERADAPTERHEADER,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.InterAdapterHeader)
  ))
_sym_db.RegisterMessage(InterAdapterHeader)

InterAdapterResponseBody = _reflection.GeneratedProtocolMessageType('InterAdapterResponseBody', (_message.Message,), dict(
  DESCRIPTOR = _INTERADAPTERRESPONSEBODY,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.InterAdapterResponseBody)
  ))
_sym_db.RegisterMessage(InterAdapterResponseBody)

InterAdapterMessage = _reflection.GeneratedProtocolMessageType('InterAdapterMessage', (_message.Message,), dict(
  DESCRIPTOR = _INTERADAPTERMESSAGE,
  __module__ = 'inter_container_pb2'
  # @@protoc_insertion_point(class_scope:voltha.InterAdapterMessage)
  ))
_sym_db.RegisterMessage(InterAdapterMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z4github.com/opencord/voltha-go/protos/inter_container'))
# @@protoc_insertion_point(module_scope)
