# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: voltha.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
import yang_options_pb2 as yang__options__pb2
import meta_pb2 as meta__pb2
import common_pb2 as common__pb2
import health_pb2 as health__pb2
import logical_device_pb2 as logical__device__pb2
import device_pb2 as device__pb2
import adapter_pb2 as adapter__pb2
import openflow_13_pb2 as openflow__13__pb2
try:
  yang__options__pb2 = openflow__13__pb2.yang__options__pb2
except AttributeError:
  yang__options__pb2 = openflow__13__pb2.yang_options_pb2
import omci_mib_db_pb2 as omci__mib__db__pb2
import omci_alarm_db_pb2 as omci__alarm__db__pb2

from meta_pb2 import *
from common_pb2 import *
from health_pb2 import *
from logical_device_pb2 import *
from device_pb2 import *
from adapter_pb2 import *
from openflow_13_pb2 import *

DESCRIPTOR = _descriptor.FileDescriptor(
  name='voltha.proto',
  package='voltha',
  syntax='proto3',
  serialized_pb=_b('\n\x0cvoltha.proto\x12\x06voltha\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x12yang_options.proto\x1a\nmeta.proto\x1a\x0c\x63ommon.proto\x1a\x0chealth.proto\x1a\x14logical_device.proto\x1a\x0c\x64\x65vice.proto\x1a\radapter.proto\x1a\x11openflow_13.proto\x1a\x11omci_mib_db.proto\x1a\x13omci_alarm_db.proto\"\x87\x01\n\x0b\x44\x65viceGroup\x12\x11\n\x02id\x18\x01 \x01(\tB\x05\xe8\xf6\xcd\x1d\x01\x12\x39\n\x0flogical_devices\x18\x02 \x03(\x0b\x32\x15.voltha.LogicalDeviceB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12*\n\x07\x64\x65vices\x18\x03 \x03(\x0b\x32\x0e.voltha.DeviceB\t\xe2\xf6\xcd\x1d\x04\n\x02id\"2\n\x0c\x44\x65viceGroups\x12\"\n\x05items\x18\x01 \x03(\x0b\x32\x13.voltha.DeviceGroup\"\x7f\n\x12\x41larmFilterRuleKey\"b\n\x12\x41larmFilterRuleKey\x12\x06\n\x02id\x10\x00\x12\x08\n\x04type\x10\x01\x12\x0c\n\x08severity\x10\x02\x12\x0f\n\x0bresource_id\x10\x03\x12\x0c\n\x08\x63\x61tegory\x10\x04\x12\r\n\tdevice_id\x10\x05:\x05\xf0\xf6\xcd\x1d\x00\"\\\n\x0f\x41larmFilterRule\x12:\n\x03key\x18\x01 \x01(\x0e\x32-.voltha.AlarmFilterRuleKey.AlarmFilterRuleKey\x12\r\n\x05value\x18\x02 \x01(\t\"H\n\x0b\x41larmFilter\x12\x11\n\x02id\x18\x01 \x01(\tB\x05\xe8\xf6\xcd\x1d\x01\x12&\n\x05rules\x18\x02 \x03(\x0b\x32\x17.voltha.AlarmFilterRule\"4\n\x0c\x41larmFilters\x12$\n\x07\x66ilters\x18\x01 \x03(\x0b\x32\x13.voltha.AlarmFilter\"I\n\x07Logging\x12(\n\x05level\x18\x01 \x01(\x0e\x32\x19.voltha.LogLevel.LogLevel\x12\x14\n\x0cpackage_name\x18\x02 \x01(\t\"^\n\x0c\x43oreInstance\x12\x1a\n\x0binstance_id\x18\x01 \x01(\tB\x05\xe8\xf6\xcd\x1d\x01\x12+\n\x06health\x18\x02 \x01(\x0b\x32\x14.voltha.HealthStatusB\x05\xe2\xf6\xcd\x1d\x00:\x05\xf8\xf6\xcd\x1d\x01\";\n\rCoreInstances\x12#\n\x05items\x18\x01 \x03(\x0b\x32\x14.voltha.CoreInstance:\x05\xf8\xf6\xcd\x1d\x01\"\xe8\x03\n\x06Voltha\x12\x16\n\x07version\x18\x01 \x01(\tB\x05\xe8\xf6\xcd\x1d\x01\x12,\n\x08\x61\x64\x61pters\x18\x02 \x03(\x0b\x32\x0f.voltha.AdapterB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12\x39\n\x0flogical_devices\x18\x03 \x03(\x0b\x32\x15.voltha.LogicalDeviceB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12*\n\x07\x64\x65vices\x18\x04 \x03(\x0b\x32\x0e.voltha.DeviceB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12\x33\n\x0c\x64\x65vice_types\x18\x05 \x03(\x0b\x32\x12.voltha.DeviceTypeB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12\x35\n\rdevice_groups\x18\x06 \x03(\x0b\x32\x13.voltha.DeviceGroupB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12\x35\n\ralarm_filters\x18\x07 \x03(\x0b\x32\x13.voltha.AlarmFilterB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12@\n\x11omci_mib_database\x18\x1c \x03(\x0b\x32\x13.omci.MibDeviceDataB\x10\xe2\xf6\xcd\x1d\x0b\n\tdevice_id\x12\x45\n\x13omci_alarm_database\x18\x1d \x03(\x0b\x32\x16.alarm.AlarmDeviceDataB\x10\xe2\xf6\xcd\x1d\x0b\n\tdevice_id:\x05\xf8\xf6\xcd\x1d\x01\"\xa4\x01\n\x10SelfTestResponse\x12\x37\n\x06result\x18\x01 \x01(\x0e\x32\'.voltha.SelfTestResponse.SelfTestResult\"P\n\x0eSelfTestResult\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0b\n\x07\x46\x41ILURE\x10\x01\x12\x11\n\rNOT_SUPPORTED\x10\x02\x12\x11\n\rUNKNOWN_ERROR\x10\x03:\x05\xf0\xf6\xcd\x1d\x00\":\n\x11OfAgentSubscriber\x12\x12\n\nofagent_id\x18\x01 \x01(\t\x12\x11\n\tvoltha_id\x18\x02 \x01(\t2\xf0-\n\rVolthaService\x12O\n\x0eUpdateLogLevel\x12\x0f.voltha.Logging\x1a\x16.google.protobuf.Empty\"\x14\x82\xd3\xe4\x93\x02\x0e\x12\x0c/api/v1/logs\x12\x44\n\tGetVoltha\x12\x16.google.protobuf.Empty\x1a\x0e.voltha.Voltha\"\x0f\x82\xd3\xe4\x93\x02\t\x12\x07/api/v1\x12u\n\x11ListCoreInstances\x12\x16.google.protobuf.Empty\x1a\x15.voltha.CoreInstances\"1\x82\xd3\xe4\x93\x02\x13\x12\x11/api/v1/instances\x8a\xf7\xcd\x1d\x07\n\x05items\x8a\xf7\xcd\x1d\x07\x12\x05items\x12S\n\x0fGetCoreInstance\x12\n.voltha.ID\x1a\x14.voltha.CoreInstance\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16/api/v1/instances/{id}\x12\x61\n\x0cListAdapters\x12\x16.google.protobuf.Empty\x1a\x10.voltha.Adapters\"\'\x82\xd3\xe4\x93\x02\x12\x12\x10/api/v1/adapters\x8a\xf7\xcd\x1d\n\n\x08\x61\x64\x61pters\x12{\n\x12ListLogicalDevices\x12\x16.google.protobuf.Empty\x1a\x16.voltha.LogicalDevices\"5\x82\xd3\xe4\x93\x02\x19\x12\x17/api/v1/logical_devices\x8a\xf7\xcd\x1d\x11\n\x0flogical_devices\x12[\n\x10GetLogicalDevice\x12\n.voltha.ID\x1a\x15.voltha.LogicalDevice\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/api/v1/logical_devices/{id}\x12r\n\x16ListLogicalDevicePorts\x12\n.voltha.ID\x1a\x14.voltha.LogicalPorts\"6\x82\xd3\xe4\x93\x02$\x12\"/api/v1/logical_devices/{id}/ports\x8a\xf7\xcd\x1d\x07\n\x05ports\x12\x83\x01\n\x14GetLogicalDevicePort\x12\x15.voltha.LogicalPortId\x1a\x13.voltha.LogicalPort\"?\x82\xd3\xe4\x93\x02.\x12,/api/v1/logical_devices/{id}/ports/{port_id}\x8a\xf7\xcd\x1d\x06\n\x04port\x12\x85\x01\n\x17\x45nableLogicalDevicePort\x12\x15.voltha.LogicalPortId\x1a\x16.google.protobuf.Empty\";\x82\xd3\xe4\x93\x02\x35\"3/api/v1/logical_devices/{id}/ports/{port_id}/enable\x12\x87\x01\n\x18\x44isableLogicalDevicePort\x12\x15.voltha.LogicalPortId\x1a\x16.google.protobuf.Empty\"<\x82\xd3\xe4\x93\x02\x36\"4/api/v1/logical_devices/{id}/ports/{port_id}/disable\x12|\n\x16ListLogicalDeviceFlows\x12\n.voltha.ID\x1a\x12.openflow_13.Flows\"B\x82\xd3\xe4\x93\x02$\x12\"/api/v1/logical_devices/{id}/flows\x8a\xf7\xcd\x1d\x07\n\x05\x66lows\x8a\xf7\xcd\x1d\x07\x12\x05items\x12\x83\x01\n\x1cUpdateLogicalDeviceFlowTable\x12\x1c.openflow_13.FlowTableUpdate\x1a\x16.google.protobuf.Empty\"-\x82\xd3\xe4\x93\x02\'\"\"/api/v1/logical_devices/{id}/flows:\x01*\x12\x84\x01\n\x1dUpdateLogicalDeviceMeterTable\x12\x1b.openflow_13.MeterModUpdate\x1a\x16.google.protobuf.Empty\".\x82\xd3\xe4\x93\x02(\"#/api/v1/logical_devices/{id}/meters:\x01*\x12{\n\x1cGetMeterStatsOfLogicalDevice\x12\n.voltha.ID\x1a\x1c.openflow_13.MeterStatsReply\"1\x82\xd3\xe4\x93\x02+\x12)/api/v1/logical_devices/{id}/meters_stats\x12\x92\x01\n\x1bListLogicalDeviceFlowGroups\x12\n.voltha.ID\x1a\x17.openflow_13.FlowGroups\"N\x82\xd3\xe4\x93\x02*\x12(/api/v1/logical_devices/{id}/flow_groups\x8a\xf7\xcd\x1d\r\n\x0b\x66low_groups\x8a\xf7\xcd\x1d\x07\x12\x05items\x12\x93\x01\n!UpdateLogicalDeviceFlowGroupTable\x12!.openflow_13.FlowGroupTableUpdate\x1a\x16.google.protobuf.Empty\"3\x82\xd3\xe4\x93\x02-\"(/api/v1/logical_devices/{id}/flow_groups:\x01*\x12]\n\x0bListDevices\x12\x16.google.protobuf.Empty\x1a\x0f.voltha.Devices\"%\x82\xd3\xe4\x93\x02\x11\x12\x0f/api/v1/devices\x8a\xf7\xcd\x1d\t\n\x07\x64\x65vices\x12\x64\n\rListDeviceIds\x12\x16.google.protobuf.Empty\x1a\x0b.voltha.IDs\".\x82\xd3\xe4\x93\x02\x13\x12\x11/api/v1/deviceids\x8a\xf7\xcd\x1d\x04\n\x02id\x8a\xf7\xcd\x1d\x07\x12\x05items\x12U\n\x10ReconcileDevices\x12\x0b.voltha.IDs\x1a\x16.google.protobuf.Empty\"\x1c\x82\xd3\xe4\x93\x02\x16\"\x11/api/v1/deviceids:\x01*\x12\x45\n\tGetDevice\x12\n.voltha.ID\x1a\x0e.voltha.Device\"\x1c\x82\xd3\xe4\x93\x02\x16\x12\x14/api/v1/devices/{id}\x12J\n\x0c\x43reateDevice\x12\x0e.voltha.Device\x1a\x0e.voltha.Device\"\x1a\x82\xd3\xe4\x93\x02\x14\"\x0f/api/v1/devices:\x01*\x12W\n\x0c\x45nableDevice\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"#\x82\xd3\xe4\x93\x02\x1d\"\x1b/api/v1/devices/{id}/enable\x12Y\n\rDisableDevice\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"$\x82\xd3\xe4\x93\x02\x1e\"\x1c/api/v1/devices/{id}/disable\x12W\n\x0cRebootDevice\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"#\x82\xd3\xe4\x93\x02\x1d\"\x1b/api/v1/devices/{id}/reboot\x12W\n\x0c\x44\x65leteDevice\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"#\x82\xd3\xe4\x93\x02\x1d*\x1b/api/v1/devices/{id}/delete\x12u\n\rDownloadImage\x12\x15.voltha.ImageDownload\x1a\x15.voltha.OperationResp\"6\x82\xd3\xe4\x93\x02\x30\"+/api/v1/devices/{id}/image_downloads/{name}:\x01*\x12\x82\x01\n\x16GetImageDownloadStatus\x12\x15.voltha.ImageDownload\x1a\x15.voltha.ImageDownload\":\x82\xd3\xe4\x93\x02\x34\x12\x32/api/v1/devices/{id}/image_downloads/{name}/status\x12u\n\x10GetImageDownload\x12\x15.voltha.ImageDownload\x1a\x15.voltha.ImageDownload\"3\x82\xd3\xe4\x93\x02-\x12+/api/v1/devices/{id}/image_downloads/{name}\x12\x66\n\x12ListImageDownloads\x12\n.voltha.ID\x1a\x16.voltha.ImageDownloads\",\x82\xd3\xe4\x93\x02&\x12$/api/v1/devices/{id}/image_downloads\x12x\n\x13\x43\x61ncelImageDownload\x12\x15.voltha.ImageDownload\x1a\x15.voltha.OperationResp\"3\x82\xd3\xe4\x93\x02-*+/api/v1/devices/{id}/image_downloads/{name}\x12\x88\x01\n\x13\x41\x63tivateImageUpdate\x12\x15.voltha.ImageDownload\x1a\x15.voltha.OperationResp\"C\x82\xd3\xe4\x93\x02=\"8/api/v1/devices/{id}/image_downloads/{name}/image_update:\x01*\x12\x86\x01\n\x11RevertImageUpdate\x12\x15.voltha.ImageDownload\x1a\x15.voltha.OperationResp\"C\x82\xd3\xe4\x93\x02=\"8/api/v1/devices/{id}/image_downloads/{name}/image_revert:\x01*\x12\\\n\x0fListDevicePorts\x12\n.voltha.ID\x1a\r.voltha.Ports\".\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/devices/{id}/ports\x8a\xf7\xcd\x1d\x07\n\x05ports\x12]\n\x13ListDevicePmConfigs\x12\n.voltha.ID\x1a\x11.voltha.PmConfigs\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/api/v1/devices/{id}/pm_configs\x12n\n\x15UpdateDevicePmConfigs\x12\x11.voltha.PmConfigs\x1a\x16.google.protobuf.Empty\"*\x82\xd3\xe4\x93\x02$\"\x1f/api/v1/devices/{id}/pm_configs:\x01*\x12m\n\x0fListDeviceFlows\x12\n.voltha.ID\x1a\x12.openflow_13.Flows\":\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/devices/{id}/flows\x8a\xf7\xcd\x1d\x07\n\x05\x66lows\x8a\xf7\xcd\x1d\x07\x12\x05items\x12\x83\x01\n\x14ListDeviceFlowGroups\x12\n.voltha.ID\x1a\x17.openflow_13.FlowGroups\"F\x82\xd3\xe4\x93\x02\"\x12 /api/v1/devices/{id}/flow_groups\x8a\xf7\xcd\x1d\r\n\x0b\x66low_groups\x8a\xf7\xcd\x1d\x07\x12\x05items\x12o\n\x0fListDeviceTypes\x12\x16.google.protobuf.Empty\x1a\x13.voltha.DeviceTypes\"/\x82\xd3\xe4\x93\x02\x16\x12\x14/api/v1/device_types\x8a\xf7\xcd\x1d\x0e\n\x0c\x64\x65vice_types\x12R\n\rGetDeviceType\x12\n.voltha.ID\x1a\x12.voltha.DeviceType\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/api/v1/device_types/{id}\x12s\n\x10ListDeviceGroups\x12\x16.google.protobuf.Empty\x1a\x14.voltha.DeviceGroups\"1\x82\xd3\xe4\x93\x02\x17\x12\x15/api/v1/device_groups\x8a\xf7\xcd\x1d\x0f\n\rdevice_groups\x12\x46\n\x10StreamPacketsOut\x12\x16.openflow_13.PacketOut\x1a\x16.google.protobuf.Empty\"\x00(\x01\x12\x45\n\x10ReceivePacketsIn\x12\x16.google.protobuf.Empty\x1a\x15.openflow_13.PacketIn\"\x00\x30\x01\x12K\n\x13ReceiveChangeEvents\x12\x16.google.protobuf.Empty\x1a\x18.openflow_13.ChangeEvent\"\x00\x30\x01\x12U\n\x0eGetDeviceGroup\x12\n.voltha.ID\x1a\x13.voltha.DeviceGroup\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/device_groups/{id}\x12_\n\x11\x43reateAlarmFilter\x12\x13.voltha.AlarmFilter\x1a\x13.voltha.AlarmFilter\" \x82\xd3\xe4\x93\x02\x1a\"\x15/api/v1/alarm_filters:\x01*\x12U\n\x0eGetAlarmFilter\x12\n.voltha.ID\x1a\x13.voltha.AlarmFilter\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/alarm_filters/{id}\x12\x64\n\x11UpdateAlarmFilter\x12\x13.voltha.AlarmFilter\x1a\x13.voltha.AlarmFilter\"%\x82\xd3\xe4\x93\x02\x1f\x1a\x1a/api/v1/alarm_filters/{id}:\x01*\x12[\n\x11\x44\x65leteAlarmFilter\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"\"\x82\xd3\xe4\x93\x02\x1c*\x1a/api/v1/alarm_filters/{id}\x12_\n\x10ListAlarmFilters\x12\x16.google.protobuf.Empty\x1a\x14.voltha.AlarmFilters\"\x1d\x82\xd3\xe4\x93\x02\x17\x12\x15/api/v1/alarm_filters\x12L\n\tGetImages\x12\n.voltha.ID\x1a\x0e.voltha.Images\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/api/v1/devices/{id}/images\x12X\n\x08SelfTest\x12\n.voltha.ID\x1a\x18.voltha.SelfTestResponse\"&\x82\xd3\xe4\x93\x02 \"\x1e/api/v1/devices/{id}/self_test\x12V\n\x10GetMibDeviceData\x12\n.voltha.ID\x1a\x13.omci.MibDeviceData\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/api/v1/openomci/{id}/mib\x12]\n\x12GetAlarmDeviceData\x12\n.voltha.ID\x1a\x16.alarm.AlarmDeviceData\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/api/v1/openomci/{id}/alarm\x12s\n\rSimulateAlarm\x12\x1c.voltha.SimulateAlarmRequest\x1a\x15.voltha.OperationResp\"-\x82\xd3\xe4\x93\x02\'\"\"/api/v1/devices/{id}/simulate_larm:\x01*\x12\x43\n\tSubscribe\x12\x19.voltha.OfAgentSubscriber\x1a\x19.voltha.OfAgentSubscriber\"\x00\x42i\n\x13org.opencord.volthaB\x0cVolthaProtosZ+github.com/opencord/voltha-go/protos/voltha\xaa\x02\x16Opencord.Voltha.VolthaP\x03P\x04P\x05P\x06P\x07P\x08P\tb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,yang__options__pb2.DESCRIPTOR,meta__pb2.DESCRIPTOR,common__pb2.DESCRIPTOR,health__pb2.DESCRIPTOR,logical__device__pb2.DESCRIPTOR,device__pb2.DESCRIPTOR,adapter__pb2.DESCRIPTOR,openflow__13__pb2.DESCRIPTOR,omci__mib__db__pb2.DESCRIPTOR,omci__alarm__db__pb2.DESCRIPTOR,],
  public_dependencies=[meta__pb2.DESCRIPTOR,common__pb2.DESCRIPTOR,health__pb2.DESCRIPTOR,logical__device__pb2.DESCRIPTOR,device__pb2.DESCRIPTOR,adapter__pb2.DESCRIPTOR,openflow__13__pb2.DESCRIPTOR,])



_ALARMFILTERRULEKEY_ALARMFILTERRULEKEY = _descriptor.EnumDescriptor(
  name='AlarmFilterRuleKey',
  full_name='voltha.AlarmFilterRuleKey.AlarmFilterRuleKey',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='id', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='type', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='severity', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='resource_id', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='category', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='device_id', index=5, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=465,
  serialized_end=563,
)
_sym_db.RegisterEnumDescriptor(_ALARMFILTERRULEKEY_ALARMFILTERRULEKEY)

_SELFTESTRESPONSE_SELFTESTRESULT = _descriptor.EnumDescriptor(
  name='SelfTestResult',
  full_name='voltha.SelfTestResponse.SelfTestResult',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILURE', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOT_SUPPORTED', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_ERROR', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1595,
  serialized_end=1675,
)
_sym_db.RegisterEnumDescriptor(_SELFTESTRESPONSE_SELFTESTRESULT)


_DEVICEGROUP = _descriptor.Descriptor(
  name='DeviceGroup',
  full_name='voltha.DeviceGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='voltha.DeviceGroup.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))),
    _descriptor.FieldDescriptor(
      name='logical_devices', full_name='voltha.DeviceGroup.logical_devices', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='devices', full_name='voltha.DeviceGroup.devices', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=254,
  serialized_end=389,
)


_DEVICEGROUPS = _descriptor.Descriptor(
  name='DeviceGroups',
  full_name='voltha.DeviceGroups',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='voltha.DeviceGroups.items', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=391,
  serialized_end=441,
)


_ALARMFILTERRULEKEY = _descriptor.Descriptor(
  name='AlarmFilterRuleKey',
  full_name='voltha.AlarmFilterRuleKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ALARMFILTERRULEKEY_ALARMFILTERRULEKEY,
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\360\366\315\035\000')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=443,
  serialized_end=570,
)


_ALARMFILTERRULE = _descriptor.Descriptor(
  name='AlarmFilterRule',
  full_name='voltha.AlarmFilterRule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='voltha.AlarmFilterRule.key', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='voltha.AlarmFilterRule.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=572,
  serialized_end=664,
)


_ALARMFILTER = _descriptor.Descriptor(
  name='AlarmFilter',
  full_name='voltha.AlarmFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='voltha.AlarmFilter.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))),
    _descriptor.FieldDescriptor(
      name='rules', full_name='voltha.AlarmFilter.rules', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=666,
  serialized_end=738,
)


_ALARMFILTERS = _descriptor.Descriptor(
  name='AlarmFilters',
  full_name='voltha.AlarmFilters',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filters', full_name='voltha.AlarmFilters.filters', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=740,
  serialized_end=792,
)


_LOGGING = _descriptor.Descriptor(
  name='Logging',
  full_name='voltha.Logging',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='level', full_name='voltha.Logging.level', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='package_name', full_name='voltha.Logging.package_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=794,
  serialized_end=867,
)


_COREINSTANCE = _descriptor.Descriptor(
  name='CoreInstance',
  full_name='voltha.CoreInstance',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_id', full_name='voltha.CoreInstance.instance_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))),
    _descriptor.FieldDescriptor(
      name='health', full_name='voltha.CoreInstance.health', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\000'))),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=869,
  serialized_end=963,
)


_COREINSTANCES = _descriptor.Descriptor(
  name='CoreInstances',
  full_name='voltha.CoreInstances',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='voltha.CoreInstances.items', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=965,
  serialized_end=1024,
)


_VOLTHA = _descriptor.Descriptor(
  name='Voltha',
  full_name='voltha.Voltha',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='voltha.Voltha.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))),
    _descriptor.FieldDescriptor(
      name='adapters', full_name='voltha.Voltha.adapters', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='logical_devices', full_name='voltha.Voltha.logical_devices', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='devices', full_name='voltha.Voltha.devices', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='device_types', full_name='voltha.Voltha.device_types', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='device_groups', full_name='voltha.Voltha.device_groups', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='alarm_filters', full_name='voltha.Voltha.alarm_filters', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='omci_mib_database', full_name='voltha.Voltha.omci_mib_database', index=7,
      number=28, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\013\n\tdevice_id'))),
    _descriptor.FieldDescriptor(
      name='omci_alarm_database', full_name='voltha.Voltha.omci_alarm_database', index=8,
      number=29, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\013\n\tdevice_id'))),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1027,
  serialized_end=1515,
)


_SELFTESTRESPONSE = _descriptor.Descriptor(
  name='SelfTestResponse',
  full_name='voltha.SelfTestResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='voltha.SelfTestResponse.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SELFTESTRESPONSE_SELFTESTRESULT,
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\360\366\315\035\000')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1518,
  serialized_end=1682,
)


_OFAGENTSUBSCRIBER = _descriptor.Descriptor(
  name='OfAgentSubscriber',
  full_name='voltha.OfAgentSubscriber',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ofagent_id', full_name='voltha.OfAgentSubscriber.ofagent_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='voltha_id', full_name='voltha.OfAgentSubscriber.voltha_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1684,
  serialized_end=1742,
)

_DEVICEGROUP.fields_by_name['logical_devices'].message_type = logical__device__pb2._LOGICALDEVICE
_DEVICEGROUP.fields_by_name['devices'].message_type = device__pb2._DEVICE
_DEVICEGROUPS.fields_by_name['items'].message_type = _DEVICEGROUP
_ALARMFILTERRULEKEY_ALARMFILTERRULEKEY.containing_type = _ALARMFILTERRULEKEY
_ALARMFILTERRULE.fields_by_name['key'].enum_type = _ALARMFILTERRULEKEY_ALARMFILTERRULEKEY
_ALARMFILTER.fields_by_name['rules'].message_type = _ALARMFILTERRULE
_ALARMFILTERS.fields_by_name['filters'].message_type = _ALARMFILTER
_LOGGING.fields_by_name['level'].enum_type = common__pb2._LOGLEVEL_LOGLEVEL
_COREINSTANCE.fields_by_name['health'].message_type = health__pb2._HEALTHSTATUS
_COREINSTANCES.fields_by_name['items'].message_type = _COREINSTANCE
_VOLTHA.fields_by_name['adapters'].message_type = adapter__pb2._ADAPTER
_VOLTHA.fields_by_name['logical_devices'].message_type = logical__device__pb2._LOGICALDEVICE
_VOLTHA.fields_by_name['devices'].message_type = device__pb2._DEVICE
_VOLTHA.fields_by_name['device_types'].message_type = device__pb2._DEVICETYPE
_VOLTHA.fields_by_name['device_groups'].message_type = _DEVICEGROUP
_VOLTHA.fields_by_name['alarm_filters'].message_type = _ALARMFILTER
_VOLTHA.fields_by_name['omci_mib_database'].message_type = omci__mib__db__pb2._MIBDEVICEDATA
_VOLTHA.fields_by_name['omci_alarm_database'].message_type = omci__alarm__db__pb2._ALARMDEVICEDATA
_SELFTESTRESPONSE.fields_by_name['result'].enum_type = _SELFTESTRESPONSE_SELFTESTRESULT
_SELFTESTRESPONSE_SELFTESTRESULT.containing_type = _SELFTESTRESPONSE
DESCRIPTOR.message_types_by_name['DeviceGroup'] = _DEVICEGROUP
DESCRIPTOR.message_types_by_name['DeviceGroups'] = _DEVICEGROUPS
DESCRIPTOR.message_types_by_name['AlarmFilterRuleKey'] = _ALARMFILTERRULEKEY
DESCRIPTOR.message_types_by_name['AlarmFilterRule'] = _ALARMFILTERRULE
DESCRIPTOR.message_types_by_name['AlarmFilter'] = _ALARMFILTER
DESCRIPTOR.message_types_by_name['AlarmFilters'] = _ALARMFILTERS
DESCRIPTOR.message_types_by_name['Logging'] = _LOGGING
DESCRIPTOR.message_types_by_name['CoreInstance'] = _COREINSTANCE
DESCRIPTOR.message_types_by_name['CoreInstances'] = _COREINSTANCES
DESCRIPTOR.message_types_by_name['Voltha'] = _VOLTHA
DESCRIPTOR.message_types_by_name['SelfTestResponse'] = _SELFTESTRESPONSE
DESCRIPTOR.message_types_by_name['OfAgentSubscriber'] = _OFAGENTSUBSCRIBER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeviceGroup = _reflection.GeneratedProtocolMessageType('DeviceGroup', (_message.Message,), dict(
  DESCRIPTOR = _DEVICEGROUP,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.DeviceGroup)
  ))
_sym_db.RegisterMessage(DeviceGroup)

DeviceGroups = _reflection.GeneratedProtocolMessageType('DeviceGroups', (_message.Message,), dict(
  DESCRIPTOR = _DEVICEGROUPS,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.DeviceGroups)
  ))
_sym_db.RegisterMessage(DeviceGroups)

AlarmFilterRuleKey = _reflection.GeneratedProtocolMessageType('AlarmFilterRuleKey', (_message.Message,), dict(
  DESCRIPTOR = _ALARMFILTERRULEKEY,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmFilterRuleKey)
  ))
_sym_db.RegisterMessage(AlarmFilterRuleKey)

AlarmFilterRule = _reflection.GeneratedProtocolMessageType('AlarmFilterRule', (_message.Message,), dict(
  DESCRIPTOR = _ALARMFILTERRULE,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmFilterRule)
  ))
_sym_db.RegisterMessage(AlarmFilterRule)

AlarmFilter = _reflection.GeneratedProtocolMessageType('AlarmFilter', (_message.Message,), dict(
  DESCRIPTOR = _ALARMFILTER,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmFilter)
  ))
_sym_db.RegisterMessage(AlarmFilter)

AlarmFilters = _reflection.GeneratedProtocolMessageType('AlarmFilters', (_message.Message,), dict(
  DESCRIPTOR = _ALARMFILTERS,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmFilters)
  ))
_sym_db.RegisterMessage(AlarmFilters)

Logging = _reflection.GeneratedProtocolMessageType('Logging', (_message.Message,), dict(
  DESCRIPTOR = _LOGGING,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.Logging)
  ))
_sym_db.RegisterMessage(Logging)

CoreInstance = _reflection.GeneratedProtocolMessageType('CoreInstance', (_message.Message,), dict(
  DESCRIPTOR = _COREINSTANCE,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.CoreInstance)
  ))
_sym_db.RegisterMessage(CoreInstance)

CoreInstances = _reflection.GeneratedProtocolMessageType('CoreInstances', (_message.Message,), dict(
  DESCRIPTOR = _COREINSTANCES,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.CoreInstances)
  ))
_sym_db.RegisterMessage(CoreInstances)

Voltha = _reflection.GeneratedProtocolMessageType('Voltha', (_message.Message,), dict(
  DESCRIPTOR = _VOLTHA,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.Voltha)
  ))
_sym_db.RegisterMessage(Voltha)

SelfTestResponse = _reflection.GeneratedProtocolMessageType('SelfTestResponse', (_message.Message,), dict(
  DESCRIPTOR = _SELFTESTRESPONSE,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.SelfTestResponse)
  ))
_sym_db.RegisterMessage(SelfTestResponse)

OfAgentSubscriber = _reflection.GeneratedProtocolMessageType('OfAgentSubscriber', (_message.Message,), dict(
  DESCRIPTOR = _OFAGENTSUBSCRIBER,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.OfAgentSubscriber)
  ))
_sym_db.RegisterMessage(OfAgentSubscriber)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023org.opencord.volthaB\014VolthaProtosZ+github.com/opencord/voltha-go/protos/voltha\252\002\026Opencord.Voltha.Voltha'))
_DEVICEGROUP.fields_by_name['id'].has_options = True
_DEVICEGROUP.fields_by_name['id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))
_DEVICEGROUP.fields_by_name['logical_devices'].has_options = True
_DEVICEGROUP.fields_by_name['logical_devices']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_DEVICEGROUP.fields_by_name['devices'].has_options = True
_DEVICEGROUP.fields_by_name['devices']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_ALARMFILTERRULEKEY.has_options = True
_ALARMFILTERRULEKEY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\360\366\315\035\000'))
_ALARMFILTER.fields_by_name['id'].has_options = True
_ALARMFILTER.fields_by_name['id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))
_COREINSTANCE.fields_by_name['instance_id'].has_options = True
_COREINSTANCE.fields_by_name['instance_id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))
_COREINSTANCE.fields_by_name['health'].has_options = True
_COREINSTANCE.fields_by_name['health']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\000'))
_COREINSTANCE.has_options = True
_COREINSTANCE._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001'))
_COREINSTANCES.has_options = True
_COREINSTANCES._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001'))
_VOLTHA.fields_by_name['version'].has_options = True
_VOLTHA.fields_by_name['version']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))
_VOLTHA.fields_by_name['adapters'].has_options = True
_VOLTHA.fields_by_name['adapters']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['logical_devices'].has_options = True
_VOLTHA.fields_by_name['logical_devices']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['devices'].has_options = True
_VOLTHA.fields_by_name['devices']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['device_types'].has_options = True
_VOLTHA.fields_by_name['device_types']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['device_groups'].has_options = True
_VOLTHA.fields_by_name['device_groups']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['alarm_filters'].has_options = True
_VOLTHA.fields_by_name['alarm_filters']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['omci_mib_database'].has_options = True
_VOLTHA.fields_by_name['omci_mib_database']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\013\n\tdevice_id'))
_VOLTHA.fields_by_name['omci_alarm_database'].has_options = True
_VOLTHA.fields_by_name['omci_alarm_database']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\013\n\tdevice_id'))
_VOLTHA.has_options = True
_VOLTHA._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001'))
_SELFTESTRESPONSE.has_options = True
_SELFTESTRESPONSE._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\360\366\315\035\000'))
# @@protoc_insertion_point(module_scope)
